CoolAid = LibStub("AceAddon-3.0"):NewAddon("CoolAid", "AceEvent-3.0", "AceConsole-3.0", "LibBars-1.0")
local addon = CoolAid
local db
local anchor

local media = LibStub("LibSharedMedia-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local GetSpellInfo = _G.GetSpellInfo
local pairs = _G.pairs
local strformat = _G.string.format
local strjoin = _G.string.join
local bitband = _G.bit.band
local COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = _G.COMBATLOG_OBJECT_AFFILIATION_OUTSIDER

local cooldowns = {
	[(GetSpellInfo(1766))] = 8, -- Kick
	[(GetSpellInfo(47528))] = 10, -- Mind Freeze
	[(GetSpellInfo(6555))] = 10, -- Pummel
	[(GetSpellInfo(72))] = 12, -- Shield Bash
	[(GetSpellInfo(2139))] = 24, -- Counterspell
	[(GetSpellInfo(19647))] = 24, -- Spell Lock
	[(GetSpellInfo(6346))] = 180, -- Fear Ward
	[(GetSpellInfo(29166))] = 360, -- Innervate
	[(GetSpellInfo(2825))] = 300, -- Bloodlust
	[(GetSpellInfo(32182))] = 300, -- Heroism
	[(GetSpellInfo(64205))] = 120, -- Divine Sacrifice
	}

local durations = {
	[(GetSpellInfo(871))] = 12, -- Shield Wall
	[(GetSpellInfo(33206))] = 8, -- Pain Suppression
	[(GetSpellInfo(47788))] = 10, -- Guadian Spirit
	[(GetSpellInfo(6940))] = 12, -- Hand of Sacrifice
	[(GetSpellInfo(498))] = 12, -- Divine Protection
	[(GetSpellInfo(61336))] = 20, -- Survival Instincts
	[(GetSpellInfo(48792))] = 12, -- Icebound Fortitude
	[(GetSpellInfo(55233))] = 20, -- Vampiric Blood
	}

local defaults = {
	profile = {
		growth = true,
		texture = "default",
		scale = 1,
		width = 200,
		height = 10,
		font = "ABF",
		fontsize = 10,
		position = {},
		spells = {},
	},
}

local options = {
	type = "group",
	args = {
		toggle = {
			type = "execute",
			name = "Toggle anchors",
			desc = "Toggle the bar anchor frames",
			func = function()
					addon:ToggleAnchor()
				end,
			order = 1,
		},
		bars = {
			type = "group",
			name = "Bar settings",
			desc = "Bar settings",
			order = 10,
			args = {
				growth = {
					type = "toggle",
					name = "Grow upwards",
					desc = "Toggle bars grow upwards/downwards from anchor",
					get = function () return db.growth end,
					set = function (info, v)
						db.growth = v
						addon:UpdateAnchor()
					end,
					order = 1,
				},
				width = {
					type = "range",
					name = "Width",
					desc = "Set the width of the bars",
					get = function() return db.width end,
					set = function(info, v)
							db.width = v
							addon:UpdateAnchor()
						end,
					min = 50,
					max = 500,
					step = 5,
					isPercent = false,
					order = 2,
				},
				height = {
					type = "range",
					name = "Height",
					desc = "Set the height of the bars",
					get = function() return db.height end,
					set = function(info, v)
							db.height = v
							addon:UpdateAnchor()
						end,
					min = 10,
					max = 100,
					step = 1,
					isPercent = false,
					order = 3,
				},
				scale = {
					type = "range",
					name = "Scale",
					desc = "Set the scale of the bars",
					get = function() return db.scale end,
					set = function(info, v)
							db.scale = v
							addon:UpdateAnchor()
						end,
					min = 0.1,
					max = 5,
					step = 0.01,
					isPercent = true,
					order = 4,
				},
				texture = {
					type = "select",
					dialogControl = 'LSM30_Statusbar',
					order = 6,
					name = "Texture",
					desc = "Set the texture for the timer bars",
					values = AceGUIWidgetLSMlists.statusbar,
					get = function() return db.texture end,
					set = function(i,v)
							db.texture = v
							addon:UpdateAnchor()
						end,
				},
				font = {
					type = "select",
					dialogControl = 'LSM30_Font',
					order = 6,
					name = "Font",
					desc = "Set the bar font",
					values = AceGUIWidgetLSMlists.font,
					get = function() return db.font end,
					set = function(i, v)
							db.font = v
							addon:UpdateAnchor()
						end,
					order = 7,
				},
				fontsize = {
					type = "range",
					name = "Font Size",
					desc = "Set the bar font size",
					min = 7,
					max = 48,
					step = 1,
					get = function(info) return db.fontsize end,
					set = function(info, value)
							db.fontsize = value
							addon:UpdateAnchor()
						end,
					order = 8,
				},
			},
		},
		spells = {
			type = 'group',
			name = 'Spells',
			desc = 'Toggle spell timer displays',
			order = 20,
			args = {},
		},
	},
}

for k in pairs(cooldowns) do
	defaults.profile.spells[k] = true
	options.args.spells.args[k] = {
		type = "toggle",
		name = k,
		get = function () return db.spells[k] end,
		set = function (i,v) db.spells[k] = v end,
	}
end

for k in pairs(durations) do
	defaults.profile.spells[k] = true
	options.args.spells.args[k] = {
		type = "toggle",
		name = k,
		get = function () return db.spells[k] end,
		set = function (i,v) db.spells[k] = v end,
	}
end

function addon:OnInitialize()
	addon.db = LibStub("AceDB-3.0"):New("CoolAidDB", defaults, "Default")
	db = addon.db.profile 

	AceConfig:RegisterOptionsTable("CoolAid", options)

	addon:RegisterChatCommand("coolaid", function() LibStub("AceConfigDialog-3.0"):Open("CoolAid") end )
	local optFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CoolAid", "CoolAid")
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)

	addon.db.RegisterCallback(self, "OnProfileChanged", "UpdateAnchor")
	addon.db.RegisterCallback(self, "OnProfileCopied", "UpdateAnchor")
	addon.db.RegisterCallback(self, "OnProfileReset", "UpdateAnchor")	

	media.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateAnchor")
	media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "UpdateAnchor")

	addon.anchor = addon:NewBarGroup("CoolAid", nil, db.width, db.height)
	anchor = addon.anchor

	anchor.RegisterCallback(self, "AnchorMoved", "SavePosition")
	anchor.RegisterCallback(self, "AnchorClicked")
	anchor:SetScale(db.scale)
	anchor:SetColorAt(1.00, 0.0, 1.0, 0.0, 1)
	anchor:SetColorAt(0.50, 1.0, 0.7, 0.0, 1)
	anchor:SetColorAt(0.00, 1.0, 0.0, 0.0, 1)
	anchor:SetFlashPeriod(0)
	anchor:HideAnchor()

	addon:UpdateAnchor()
end

function addon:OnEnable()
	addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function addon:AnchorClicked(callback, group, button)
	if button == "RightButton" then
		group:HideAnchor()
		group:Lock()
	end
end

function addon:ToggleAnchor()
	if anchor.button:IsVisible() then
		anchor:HideAnchor()
		anchor:Lock()
	else
		anchor:ShowAnchor()
		anchor:Unlock()
	end
end

function addon:SavePosition()
	local p,_,pr,px,py = anchor:GetPoint()
	db.position.p = p
	db.position.pr = pr
	db.position.px = px
	db.position.py = py
end

function addon:UpdateAnchor()
	anchor:ClearAllPoints()
	anchor:SetPoint(db.position.p or "CENTER", UIParent, db.position.pr or "CENTER", db.position.px or 0, db.position.py or 0)
	anchor:SetTexture(media:Fetch("statusbar", db.texture))
	anchor:SetFont(media:Fetch("font", db.font), db.fontsize)
	anchor:SetWidth(db.width)
	anchor:SetHeight(db.height)
	anchor:ReverseGrowth(db.growth)
end

function addon:IsGroup(flags)
	local result = (bitband(flags, COMBATLOG_OBJECT_AFFILIATION_OUTSIDER) == 0)
	return result
end

function addon:GetIcon(id)
	local name, rank, icon = GetSpellInfo(id)
	return icon
end


function addon:COMBAT_LOG_EVENT_UNFILTERED(_,timestamp,event,srcGUID,srcName,srcFlags,dstGUID,dstName,dstFlags,spellID,spellName,_,extraID,extraName)
	if (event=="SPELL_CAST_SUCCESS" or event=="SPELL_INTERRUPT") and addon:IsGroup(srcFlags) and (cooldowns[spellName]) and db.spells[spellName] then
		local id = strjoin("-",srcGUID,spellID)
		local icon = addon:GetIcon(spellID)	
		local time = cooldowns[spellName]	
		local text
		if event=="SPELL_INTERRUPT" then 
			text = strformat("%s: %s (%s)", srcName, spellName, extraName)
		elseif dstName then
			text = strformat("%s: %s (%s)", srcName, spellName, dstName)
		else
			text = strformat("%s: %s", srcName, spellName)
		end
		anchor:NewTimerBar(id, text, time, time, icon)
	elseif (event=="SPELL_AURA_APPLIED" or event=="SPELL_AURA_REMOVED") and addon:IsGroup(dstFlags) and (durations[spellName]) and db.spells[spellName] then
		local id = strjoin("-",dstGUID,spellID)
		if event=="SPELL_AURA_REMOVED" then
			local bar = anchor:GetBar(id)
			if bar then bar:Fade() end
		else
			local icon = addon:GetIcon(spellID)	
			local time = durations[spellName]
			local text = strformat("%s: %s", dstName, spellName) 
			anchor:NewTimerBar(id, text, time, time, icon)
		end
	end
end

